==> InfiniTec.Exchange_0.96_Source.zip: ChangeNotifyDictionary.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.Diagnostics;
using InfiniTec.Exchange.WebDav;

namespace InfiniTec.Exchange
{
    class ChangeNotifyDictionary<TKey, TValue> : KeyedCollection<TKey, TValue>, IDictionary<TKey, TValue>
        where TValue : ICollectionEntry<TKey>
    {
        private TypedItemBase _Item;

        public TypedItemBase Item
        {
            [DebuggerStepThrough]
            get { return _Item; }
            [DebuggerStepThrough]
            set { _Item = value; }
        }

        protected override TKey GetKeyForItem(TValue item)
        {
            return item.GetKey();
        }

        public void Save()
        {
            foreach (TValue item in Items)
            {
                item.Save();
            }
        }

        protected override void ClearItems()
        {
            foreach (TValue item in Items)
            {
                item.Delete();
            }
            base.ClearItems();
        }

        protected override void InsertItem(int index, TValue item)
        {
            item.SetItem(_Item);
            base.InsertItem(index, item);
        }

        protected override void SetItem(int index, TValue item)
        {
            item.SetItem(_Item);
            base.SetItem(index, item);
        }
        protected override void RemoveItem(int index)
        {
            Items[index].Delete();
            base.RemoveItem(index);
        }

        public ChangeNotifyDictionary(TypedItemBase item)
        {
            _Item = item;
        }

        #region IDictionary<TKey,TValue> Members

        public void Add(TKey key, TValue value)
        {
            Dictionary.Add(key, value);
        }

        public bool ContainsKey(TKey key)
        {
            return Dictionary.ContainsKey(key);
        }

        public ICollection<TKey> Keys
        {
            get { return Dictionary.Keys; }
        }

        public bool TryGetValue(TKey key, out TValue value)
        {
            return Dictionary.TryGetValue(key, out value);
        }

        public ICollection<TValue> Values
        {
            get { return Dictionary.Values; }
        }

        TValue IDictionary<TKey, TValue>.this[TKey key]
        {
            get
            {
                return Dictionary[key];
            }
            set
            {
                Dictionary[key] = value;
            }
        }

        #endregion

        #region ICollection<KeyValuePair<TKey,TValue>> Members

        public void Add(KeyValuePair<TKey, TValue> item)
        {
            Dictionary.Add(item);
        }

        public bool Contains(KeyValuePair<TKey, TValue> item)
        {
            return Dictionary.Contains(item);
        }

        public void CopyTo(KeyValuePair<TKey, TValue>[] array, int arrayIndex)
        {
            Dictionary.CopyTo(array, arrayIndex);
        }

        public bool IsReadOnly
        {
            get { return Dictionary.IsReadOnly; }
        }

        public bool Remove(KeyValuePair<TKey, TValue> item)
        {
            return Dictionary.Remove(item);
        }

        #endregion

        #region IEnumerable<KeyValuePair<TKey,TValue>> Members

        public new IEnumerator<KeyValuePair<TKey, TValue>> GetEnumerator()
        {
            return Dictionary.GetEnumerator();
        }

        #endregion
    }
}

==> InfiniTec.Exchange_0.97_Source.zip: ChangeNotifyDictionary.cs <==
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.ObjectModel;
using System.Diagnostics;
using InfiniTec.Exchange.WebDav;

namespace InfiniTec.Exchange
{
    class ChangeNotifyDictionary<TKey, TValue> : KeyedCollection<TKey, TValue>, IDictionary<TKey, TValue>
        where TValue : ICollectionEntry<TKey>
    {
        private Item _Item;

        public Item Item
        {
            [DebuggerStepThrough]
            get { return _Item; }
            [DebuggerStepThrough]
            set { _Item = value; }
        }

        protected override TKey GetKeyForItem(TValue item)
        {
            return item.GetKey();
        }

        public void Save()
        {
            foreach (TValue item in Items)
            {
                item.Save();
            }
        }

        protected override void ClearItems()
        {
            foreach (TValue item in Items)
            {
                item.Delete();
            }
            base.ClearItems();
        }

        protected override void InsertItem(int index, TValue item)
        {
            item.SetItem(_Item);
            base.InsertItem(index, item);
        }

        protected override void SetItem(int index, TValue item)
        {
            item.SetItem(_Item);
            base.SetItem(index, item);
        }
        protected override void RemoveItem(int index)
        {
            Items[index].Delete();
            base.RemoveItem(index);
        }

        public ChangeNotifyDictionary(Item item)
        {
            _Item = item;
        }

        #region IDictionary<TKey,TValue> Members

        public void Add(TKey key, TValue value)
        {
            Dictionary.Add(key, value);
        }

        public bool ContainsKey(TKey key)
        {
            return Dictionary.ContainsKey(key);
        }

        public ICollection<TKey> Keys
        {
            get { return Dictionary.Keys; }
        }

        public bool TryGetValue(TKey key, out TValue value)
        {
            return Dictionary.TryGetValue(key, out value);
        }

        public ICollection<TValue> Values
        {
            get { return Dictionary.Values; }
        }

        TValue IDictionary<TKey, TValue>.this[TKey key]
        {
            get
            {
                return Dictionary[key];
            }
            set
            {
                Dictionary[key] = value;
            }
        }

        #endregion

        #region ICollection<KeyValuePair<TKey,TValue>> Members

        public void Add(KeyValuePair<TKey, TValue> item)
        {
            Dictionary.Add(item);
        }

        public bool Contains(KeyValuePair<TKey, TValue> item)
        {
            return Dictionary.Contains(item);
        }

        public void CopyTo(KeyValuePair<TKey, TValue>[] array, int arrayIndex)
        {
            Dictionary.CopyTo(array, arrayIndex);
        }

        public bool IsReadOnly
        {
            get { return Dictionary.IsReadOnly; }
        }

        public bool Remove(KeyValuePair<TKey, TValue> item)
        {
            return Dictionary.Remove(item);
        }

        #endregion

        #region IEnumerable<KeyValuePair<TKey,TValue>> Members

        public new IEnumerator<KeyValuePair<TKey, TValue>> GetEnumerator()
        {
            return Dictionary.GetEnumerator();
        }

        #endregion
    }
}
